@@ -110,6 +110,13 @@ register."
   :group 'gnorb-gnus
   :type 'boolean)
 
+(defcustom gnorb-gnus-tick-all-tracked-messages nil
+  "When non-nil, add the tick mark to all tracked messages.
+This happens only once, at the time the association is created.
+Ticks can be safely removed later."
+  :group 'gnorb-gnus
+  :type 'boolean)
+
 (defcustom gnorb-gnus-summary-mark-format-letter "g"
   "Format letter to be used as part of your
   `gnus-summary-line-format', to indicate in the *Summary* buffer
@@ -237,66 +244,6 @@ save them into `gnorb-tmp-dir'."
 	    (when (or capture-p store)
 	      (push filename gnorb-gnus-capture-attachments))))))))
 
-;;; Make the above work in the capture process
-
-(defun gnorb-gnus-capture-attach ()
-  (when (and (or gnorb-gnus-capture-always-attach
-		 (org-capture-get :gnus-attachments))
-	     (with-current-buffer
-		 (org-capture-get :original-buffer)
-	       (memq major-mode '(gnus-summary-mode gnus-article-mode))))
-    (require 'org-attach)
-    (setq gnorb-gnus-capture-attachments nil)
-    (gnorb-gnus-collect-all-attachments t)
-    (map-y-or-n-p
-     (lambda (a)
-       (format "Attach %s to capture heading? "
-	       (file-name-nondirectory a)))
-     (lambda (a) (org-attach-attach a nil 'mv))
-     gnorb-gnus-capture-attachments
-     '("file" "files" "attach"))
-    (setq gnorb-gnus-capture-attachments nil)))
-
-(defun gnorb-gnus-capture-save-text ()
-  (when (and gnorb-gnus-copy-message-text
-	     (with-current-buffer
-		 (org-capture-get :original-buffer)
-	       (memq major-mode '(gnus-summary-mode gnus-article-mode))))
-    (save-window-excursion
-      (set-buffer (org-capture-get :original-buffer))
-      (gnus-with-article-buffer
-	(article-goto-body)
-	(if (numberp gnorb-gnus-copy-message-text)
-	    (progn
-	      (copy-to-register
-	       gnorb-gnus-copy-message-text
-	       (point) (point-max))
-	      (message "Message text copied to register %c"
-		       gnorb-gnus-copy-message-text))
-	  (kill-new (buffer-substring (point) (point-max)))
-	  (message "Message text copied to kill ring"))))))
-
-(add-hook 'org-capture-mode-hook 'gnorb-gnus-capture-attach)
-(add-hook 'org-capture-mode-hook 'gnorb-gnus-capture-save-text)
-
-(defvar org-note-abort)
-
-(defun gnorb-gnus-capture-abort-cleanup ()
-  (with-no-warnings ; For `org-note-abort'
-    (when (and org-note-abort
-	       (or gnorb-gnus-capture-always-attach
-		   (org-capture-get :gnus-attachments)))
-     (condition-case nil
-	 (progn (org-attach-delete-all)
-		(setq abort-note 'clean)
-		;; remove any gnorb-mail-header values here
-		)
-       (error
-	(setq abort-note 'dirty))))))
-
-(add-hook 'org-capture-prepare-finalize-hook
-	  'gnorb-gnus-capture-abort-cleanup)
-
 ;;; Storing, removing, and acting on Org headers in messages.
 
 (defvar gnorb-gnus-message-info nil
@@ -528,14 +475,14 @@ work."
 
 ;;;###autoload
 (defun gnorb-gnus-incoming-do-todo (arg &optional id)
-  "Call this function from a received gnus message to store a
-link to the message, prompt for a related Org heading, visit the
-heading, and trigger an action on it \(see
-`gnorb-org-trigger-actions'\).
+  "Use the message under point to trigger an action on an Org heading.
+This function stores a link to the message, prompts for a related
+Org heading, visits the heading, and triggers an action on
+it (see `gnorb-org-trigger-actions').
 
-If you've set up message tracking \(with
-`gnorb-tracking-initialize'\), Gnorb can guess which Org heading
-you probably want to trigger, which can save some time. It does
+If you've set up message tracking (with
+`gnorb-tracking-initialize'), Gnorb can guess which Org heading
+you probably want to trigger, which can save some time.  It does
 this by looking in the References header, and seeing if any of
 the messages referenced there are already being tracked by any
 headings.
@@ -646,6 +593,8 @@ you'll stay in the Gnus summary buffer."
 		(message "Message text copied to kill ring"))))
 	  (with-current-buffer buf
 	    (dolist (a articles)
+	      (when gnorb-gnus-tick-all-tracked-messages
+		(gnus-summary-mark-article a gnus-ticked-mark))
 	      (gnus-summary-update-article a))))
       (error
        ;; If these are left populated after an error, it plays hell
@@ -686,6 +635,9 @@ reply."
 	(let ((ret (make-marker)))
 	  (setq gnorb-window-conf (current-window-configuration))
 	  (move-marker gnorb-return-marker (point))
+	  (when gnorb-gnus-tick-all-tracked-messages
+	    (gnus-summary-mark-article art-no gnus-ticked-mark))
+	  (gnus-summary-update-article art-no)
 	  ;; Assume the first heading is the one we want.
 	  (gnorb-registry-make-entry
 	   msg-id from subject targ group)
@@ -708,20 +660,31 @@ reply."
 (defun gnorb-gnus-search-messages (str persist &optional head-text ret)
   "Initiate a search for gnus message links in an org subtree.
 The arg STR can be one of two things: an Org heading id value
-\(IDs should be prefixed with \"id+\"\), in which case links will
+\(IDs should be prefixed with \"id+\"), in which case links will
 be collected from that heading, or a string corresponding to an
 Org tags search, in which case links will be collected from all
 matching headings.
 
 In either case, once a collection of links have been made, they
 will all be displayed in an ephemeral group on the \"nngnorb\"
-server. There must be an active \"nngnorb\" server for this to
-work."
+server.  There must be an active \"nngnorb\" server for this to
+work.
+
+If PERSIST is non-nil, make a permanent group, and offer
+HEAD-TEXT, if present, as its name.  Otherwise create an
+ephemeral one, with RET as the value of its quit-config."
   (interactive)
   (require 'nnir)
   (let* ((nnir-address
 	  (or (catch 'found
-		(dolist (s gnus-server-method-cache)
+		;; Try very hard to find the server.
+		(when (assoc 'nngnorb gnus-secondary-select-methods)
+		  (throw 'found
+			 (format
+			  "nngnorb:%s"
+			  (nth 1 (assoc 'nngnorb
+					gnus-secondary-select-methods)))))
+		(dolist (s (append gnus-server-alist gnus-server-method-cache))
 		  (when (eq 'nngnorb (cadr s))
 		    (throw 'found (car s)))))
 	      (user-error
@@ -733,8 +696,10 @@ work."
 		 (concat "gnorb-" str)))
 	 (method (list 'nnir nnir-address))
 	 (spec (list
-		(cons 'nnir-specs (list (cons 'nnir-query-spec `((query . ,str)))
-					(cons 'nnir-group-spec `((,nnir-address ,(list name))))))
+		(cons 'nnir-specs (list (cons 'nnir-query-spec
+					      `((query . ,str)))
+					(cons 'nnir-group-spec
+					      `((,nnir-address ,(list name))))))
 		(cons 'nnir-artlist nil))))
     (if persist
 	(progn
