@@ -89,6 +89,16 @@ information about the message from which we're triggering."
   :type 'list
   :package-version '(gnorb . "1.1.3"))
 
+(defcustom gnorb-org-log-add-link t
+  "When non-nil, add a message link in a heading's LOGBOOK.
+When triggering an Org heading from a message, and adding a log
+note, the message id will be added to the text of the log note.
+When later viewing the messages, call `gnorb-org-view' with point
+on a particular logbook item to automatically go to the linked
+message."
+  :group 'gnorb-org
+  :type 'boolean)
+
 (defcustom gnorb-org-msg-id-key "GNORB_MSG_ID"
   "The name of the org property used to store the Message-IDs
   from relevant messages. This is no longer used, and will be
@@ -242,8 +252,17 @@ See the docstring of `gnorb-org-handle-mail' for details."
    (user-mail-address
     (string-match-p user-mail-address addr))))
 
+;; FIXME: Why did I break this off from
+;; `gnorb-org-extract-mail-stuff'?  It's only called from there, and
+;; it's confusing to have them separate.
 (defun gnorb-org-extract-mail-tracking (assoc-msg-ids &optional arg region)
-
+  "Return tracked mail links for the current Org subtree.
+ASSOC-MSG-IDS is a list of message-ids that have already been
+determined to be tracked by the subtree.  Return the most recent
+of these, as a candidate for composing a reply.  If there are no
+tracked messages, or if ARG (a prefix arg from earlier) is
+non-nil, ignore these tracked ids and instead scan the
+subtree (or REGION) for links, and use those instead."
   (let* ((all-links (gnorb-org-extract-links nil region))
 	 ;; The latest (by the creation-time registry key) of all the
 	 ;; tracked messages that were not sent by our user.
@@ -254,8 +273,7 @@ See the docstring of `gnorb-org-handle-mail' for details."
 	      (cl-remove-if-not
 	       (lambda (m)
 		 (let ((from (car (gnus-registry-get-id-key m 'sender))))
-		   (and from
-			(null (gnorb-user-address-match-p from)))))
+		   (not (and from (gnorb-user-address-match-p from)))))
 	       assoc-msg-ids)
 	      (lambda (r l)
 		(time-less-p
@@ -274,7 +292,7 @@ See the docstring of `gnorb-org-handle-mail' for details."
      ;; Otherwise ignore the other links in the subtree, and return
      ;; the latest message.
      (msg-id-link
-      `(gnus ,msg-id-link)))))
+      `((gnus ,msg-id-link))))))
 
 (defvar message-beginning-of-line)
 
@@ -447,15 +465,15 @@ composed.  FILE is a file to attach to the message."
 	  (when file
 	    (setq attachments (cons file attachments)))
 	  (when (fboundp 'ebdb-org-retrieve)
-	    (dolist (e (alist-get 'ebdb links))
+	    (dolist (e e-recs)
 	      (dolist (r (ebdb-org-retrieve e))
 		(let ((m (ebdb-dwim-mail r)))
 		  (when m
 		    (push m mails))))))
-	  (dolist (b (alist-get 'bbdb links))
+	  (dolist (b b-recs)
 	    (let ((m (ebdb-mail-address
 		      (car (bbdb-message-search
-			    (org-link-unescape r))))))
+			    (org-link-unescape b))))))
 	      (when m
 		(push m mails))))
 	  (when (and b-recs
@@ -593,19 +611,92 @@ default set of parameters."
   :group 'gnorb-org
   :type 'boolean)
 
-(defun gnorb-org-capture-collect-link ()
+(defun gnorb-org-capture-function ()
+  "Do various things after starting the capture process.
+Currently includes:
+
+1. Offering to move all the attachments from the message we
+captured from onto the Org heading being captured.
+
+2. Possibly saving a link to wherever we came from (see
+`gnorb-org-capture-collect-link-p').
+
+3. Possibly saving the text of the message we captured from (see
+`gnorb-gnus-copy-message-text').
+
+4. Possibly ticking the message we captured from (see
+`gnorb-gnus-tick-all-tracked-messages')."
   (when gnorb-org-capture-collect-link-p
     (let ((buf (org-capture-get :original-buffer)))
       (when buf
 	(with-current-buffer buf
 	  (when (memq major-mode '(gnus-summary-mode
 				   gnus-article-mode
-				   bbdb-mode))
-	    (call-interactively 'org-store-link)))))))
-
-(add-hook 'org-capture-mode-hook 'gnorb-org-capture-collect-link)
 
 ;;; Agenda/BBDB popup stuff
+				   bbdb-mode
+				   ebdb-mode))
+	    (call-interactively 'org-store-link))))))
+  (when (with-current-buffer
+	    (org-capture-get :original-buffer)
+	  (memq major-mode '(gnus-summary-mode gnus-article-mode)))
+    ;; This part needs to happen in the capture buffer.
+    (when (or gnorb-gnus-capture-always-attach
+	      (org-capture-get :gnus-attachments))
+      (require 'org-attach)
+      (setq gnorb-gnus-capture-attachments nil)
+      (gnorb-gnus-collect-all-attachments t)
+      (map-y-or-n-p
+       (lambda (a)
+	 (format "Attach %s to capture heading? "
+		 (file-name-nondirectory a)))
+       (lambda (a) (org-attach-attach a nil 'mv))
+       gnorb-gnus-capture-attachments
+       '("file" "files" "attach"))
+      (setq gnorb-gnus-capture-attachments nil))
+
+    ;; This part happens in the original summary/article buffer.
+    (save-window-excursion
+      (set-buffer (org-capture-get :original-buffer))
+      (let ((art-no (gnus-summary-article-number)))
+
+	(when gnorb-gnus-copy-message-text
+	  (gnus-with-article-buffer
+	    (article-goto-body)
+	    (if (numberp gnorb-gnus-copy-message-text)
+		(progn
+		  (copy-to-register
+		   gnorb-gnus-copy-message-text
+		   (point) (point-max))
+		  (message "Message text copied to register %c"
+			   gnorb-gnus-copy-message-text))
+	      (kill-new (buffer-substring (point) (point-max)))
+	      (message "Message text copied to kill ring"))))
+
+	(when gnorb-gnus-tick-all-tracked-messages
+	  (gnus-summary-mark-article art-no gnus-ticked-mark))
+
+	(gnus-summary-update-article art-no)))))
+
+(add-hook 'org-capture-mode-hook 'gnorb-org-capture-function)
+
+(defvar org-note-abort)
+
+(defun gnorb-org-capture-abort-cleanup ()
+  (with-no-warnings ;; For `org-note-abort'
+    (when (and org-note-abort
+	       (or (bound-and-true-p gnorb-gnus-capture-always-attach)
+		   (org-capture-get :gnus-attachments)))
+      (condition-case nil
+	  (progn (org-attach-delete-all)
+		 (setq abort-note 'clean)
+		 ;; remove any gnorb-mail-header values here
+		 )
+	(error
+	 (setq abort-note 'dirty))))))
+
+(add-hook 'org-capture-prepare-finalize-hook
+	  'gnorb-org-capture-abort-cleanup)
 
 (defcustom gnorb-org-agenda-popup-bbdb nil
   "Should Agenda tags search pop up a BBDB buffer with matching
